@@ -4,17 +4,22 @@ Logs handling.
 from __future__ import absolute_import
 
 import cgitb
+import inspect
 import logging
 import logging.config
+import platform
 import re
 import sys
 import threading
 import typing as tp
+from enum import Enum
+from typing import Any, Dict, Optional, Text
 
 import attr
 import structlog
 import structlog.dev
 
+from applitools.common.utils import json_utils
 from applitools.common.utils.general_utils import get_env_with_prefix
 
 __all__ = ("StdoutLogger", "FileLogger")
@@ -27,6 +32,45 @@ _frames_regex = re.compile(
 )
 
 
+class Stage(Enum):
+    GENERAL = "GENERAL"
+    OPEN = "OPEN"
+    CHECK = "CHECK"
+    CLOSE = "CLOSE"
+    RENDER = "RENDER"
+    RESOURCE_COLLECTION = "RESOURCE_COLLECTION"
+    LOCATE = "LOCATE"
+    OCR = "OCR"
+
+
+def create_message_from_log(
+    agent_id,  # type: Text
+    test_id,  # type: Text
+    stage,  # type: Stage
+    data,  # type: Dict[Text, Any]
+    type=None,  # type: Optional[Text]
+    methods_back=3,  # type: int
+):
+    # type: (...) -> Text
+    d = {
+        "agentId": agent_id,
+        "testId": test_id,
+        "stage": stage,
+        "threadId": threading.current_thread().name,
+        "stackTrace": [inspect.stack()[i][3] for i in range(2, methods_back + 1)],
+        "pythonVersion": "{} {}".format(
+            platform.python_implementation(),
+            platform.python_version(),
+        ),
+        "platformName": platform.platform(),
+    }
+    if type:
+        d["type"] = type
+
+    d.update(data)
+    return json_utils.to_json(d)
+
+
 class StdoutLogger(object):
     """
     A simple logger class for printing to STDOUT.
